var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

import React, { useContext } from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import { tagPropType, mapToCssModules } from './Shared/helper.js';
import { Context } from './CPagination';

//component - CoreUI / CPaginationLink

var CPaginationLink = function CPaginationLink(props) {
  var Tag = props.tag,
      className = props.className,
      cssModule = props.cssModule,
      innerRef = props.innerRef,
      next = props.next,
      previous = props.previous,
      onClick = props.onClick,
      type = props.type,
      n = props.n,
      attributes = _objectWithoutProperties(props, ['tag', 'className', 'cssModule', 'innerRef', 'next', 'previous', 'onClick', 'type', 'n']);

  var context = useContext(Context);

  var linkClick = function linkClick(e) {
    onClick && onClick(e, type, n);
    context.paginationClick && context.paginationClick(e, type, n);
  };

  //render

  var classes = mapToCssModules(classNames(className, 'page-link'), cssModule);

  var defaultAriaLabel = void 0;
  if (previous) {
    defaultAriaLabel = 'Previous';
  } else if (next) {
    defaultAriaLabel = 'Next';
  }
  var ariaLabel = props['aria-label'] || defaultAriaLabel;

  var defaultCaret = void 0;
  if (previous) {
    defaultCaret = React.createElement(
      React.Fragment,
      null,
      '\u2039'
    );
  } else if (next) {
    defaultCaret = React.createElement(
      React.Fragment,
      null,
      '\u203A'
    );
  }

  var children = props.children;

  if (children && Array.isArray(children) && children.length === 0) {
    children = null;
  }

  if (!attributes.href && Tag === 'a') {
    Tag = 'button';
  }

  if (previous || next) {
    children = [React.createElement(
      'span',
      {
        'aria-hidden': 'true',
        key: 'caret'
      },
      children || defaultCaret
    ), React.createElement(
      'span',
      {
        className: 'sr-only',
        key: 'sr'
      },
      ariaLabel
    )];
  }

  return React.createElement(
    Tag,
    _extends({}, attributes, {
      className: classes,
      onClick: linkClick,
      'aria-label': ariaLabel,
      ref: innerRef
    }),
    children
  );
};

CPaginationLink.propTypes = process.env.NODE_ENV !== "production" ? {
  tag: tagPropType,
  children: PropTypes.node,
  className: PropTypes.string,
  cssModule: PropTypes.object,
  //
  innerRef: PropTypes.oneOfType([PropTypes.object, PropTypes.func, PropTypes.string]),
  next: PropTypes.bool,
  type: PropTypes.string,
  n: PropTypes.number,
  previous: PropTypes.bool,
  onClick: PropTypes.func,
  'aria-label': PropTypes.string
} : {};

CPaginationLink.defaultProps = {
  tag: 'a'
};

export default CPaginationLink;